#include <stdio.h>
#include <math.h>
#include <float.h>
#include <stdlib.h>

#define TRUE 1
#define FALSE 0
#define NO_FEATURES  2
#define MAX_SIZE 100

struct record {
	float feature[ NO_FEATURES ];
	int cluster_member;    /* -1 means item was purged by algorithm */
} data[ MAX_SIZE ];

struct cl {
	int active;       /* FALSE--not active; TRUE--active */
	int size;
	float centroid[ NO_FEATURES ];
	int mod;          /* FALSE not modified; TRUE--modified */
} cluster[ MAX_SIZE + 1];   /* one extra is dummy cluster */

int no_clusters,              /* desired number of clusters */
	 iter_start,               /* max number of iterations at start */
	 local_flag,
	 data_size;                /* number of patterns */

void forgy( void ),
	  compute_centroid( int ),
	  find_nearest_seed_pts( void ),
	  dump_data( void ),
	  dump_cluster( void );
float distsq( struct record s1, struct cl s2 );

main()
{
	int i, j, ans;
	FILE *ifp;
	char filename[ 100 ];

	for ( ; ; ) {
		printf( "\n\n\n0--quit" );
		printf( "\n1--enter data from keyboard" );
		printf( "\n2--get data from a file" );
		printf( "\n3--reuse old data" );
		printf( "\nYour choice: " );
		scanf( "%d", &ans );

		if ( ans == 0 )
			return EXIT_SUCCESS;

		if ( ans == 1 )
			ifp = stdin;
		else if ( ans == 2 ) {
			printf( "\nfilename: " );
			scanf( "%s", filename );
			if ( ( ifp = fopen( filename, "r" ) ) == NULL ) {
				printf( "unable to open file %s\n", filename );
				continue;
			}
		}

		if ( ans == 1 )
			printf( "\n\nEnter data now (EOF to quit)\n" );

		if ( ans == 1 || ans == 2 )
			for ( data_size = 0;
					fscanf( ifp, "%f", &data[ data_size ].feature[ 0 ] )
						!= EOF;
					data_size++ )
				for ( i = 1; i < NO_FEATURES; i++ )
					fscanf( ifp, "%f", &data[ data_size ].feature[ i ] );

		printf( "\ndesired number of clusters: " );
		scanf( "%d", &no_clusters );

		/* Initial seed points are first no_clusters patterns.
			Each forms a cluster by itself.
			The clusters are named 0, 1, ...
		 */
		for ( i = 0; i < no_clusters; i++ ) {
			data[ i ].cluster_member = i;
			cluster[ i ].active = TRUE;
			cluster[ i ].size = 1;
			for ( j = 0; j < NO_FEATURES; j++ )
				cluster[ i ].centroid[ j ] = data[ i ].feature[ j ];
		}

		for ( ; i < data_size; i++ ) {
			data[ i ].cluster_member = MAX_SIZE;  /* dummy cluster */
			cluster[ i ].size = 1;
		}

		dump_data();
/*		dump_cluster(); */

		forgy();
/*		dump_cluster(); */
	}
}

void forgy( void )
{
	do {
		local_flag = FALSE;
		find_nearest_seed_pts();
		dump_data();
/*		dump_cluster(); */
	} while ( local_flag );
}

void find_nearest_seed_pts( void )
{
	int i, ind, j, t;
	float max, new;

	for ( i = 0; i < data_size; i++ )
		cluster[ i ].mod = FALSE;

   for ( i = 0; i < data_size; i++ )
      if ( ( t = data[ i ].cluster_member ) != -1 ) {
			max = FLT_MAX;
			for ( j = 0; j < data_size; j++ )
				if ( cluster[ j ].active
					  && ( new = distsq( data[ i ], cluster[ j ] ) ) < max ) {
               max = new;
               ind = j;
            }
         if ( t != ind ) {
            local_flag = TRUE;
				cluster[ ind ].size++;
            cluster[ t ].size--;
            data[ i ].cluster_member = ind;
            cluster[ ind ].mod = cluster[ t ].mod = TRUE;
			}
		}

	if ( local_flag )
		for ( i = 0; i < data_size; i++ )
			if ( cluster[ i ].mod )
				compute_centroid( i );
}

void compute_centroid( int i )
{
	int j, count, t;
	float center;

	for ( j = 0; j < NO_FEATURES; j++ ) {
		center = 0.0;
		count = 0;
		for ( t = 0; t < data_size; t++ )
			if ( data[ t ].cluster_member == i ) {
				center += data[ t ].feature[ j ];
				count++;
			}
		cluster[ i ].centroid[ j ] = center/count;
	}
}

float distsq( struct record s1, struct cl s2 )
{
	float dist = 0.0, diff;
	int i;

	for ( i = 0; i < NO_FEATURES; i++ ) {
		diff = s1.feature[ i ] - s2.centroid[ i ];
		dist += diff * diff;
	}

	return dist;
}

void dump_data( void )
{
	int i, j;

	printf( "\n\nDumping data array\n\n" );

	for ( i = 0; i < data_size; i++ ) {
		printf( "i = %d\n", i );
		for ( j = 0; j < NO_FEATURES; j++ )
			printf( "feature %d = %f\n", j, data[ i ].feature[ j ] );
		printf( "cluster member = %d\n", data[ i ].cluster_member );
	}
}

void dump_cluster( void )
{
	int i, j;

	printf( "\n\nDumping cluster array\n\n " );

	for ( i = 0; i < data_size; i++ ) {
		printf( "i = %d\n", i );
		printf( "active flag = %d\n", cluster[ i ].active );
		printf( "size = %d\n", cluster[ i ].size );
		for ( j = 0; j < NO_FEATURES; j++ )
			printf( "centroid %d = %f\n", j, cluster[ i ].centroid[ j ] );
		printf( "mod flag = %d\n", cluster[ i ].mod );
	}
}
