#include <stdio.h>
#include <stdlib.h>

#define MAXSIZE 50
#define NUMB_FIRES 1000

int no_nodes;
int weight[ MAXSIZE ][ MAXSIZE ];
void store_pat( signed char pattern[][ MAXSIZE + 1], int no_pats );
void convert_pat( signed char *p );
void get_pattern( signed char *retr_pattern );
void print_weights( void );
void test_stable( signed char *retr_pattern );
void choose_fire_seq( signed char *retr_pattern );

main()
{
	signed char pattern[ MAXSIZE + 1 ][ MAXSIZE + 1 ];
	signed char retr_pattern[ MAXSIZE + 1 ];
	int i;
	char resp[ 2 ];

	printf( "How many nodes? " );
	scanf( "%d", &no_nodes );
	for ( i = 0; ; i++ ) {
		printf( "\nInput next pattern to store\n"
				  "(use + - notation, or start the line with ! to stop): " );
		scanf( "%s", pattern[ i ] );
		if ( *pattern[ i ] == '!' )
			break;
		convert_pat( pattern[ i ] );
	}
	printf( "\nStoring patterns\n" );
	store_pat( pattern, i );
	print_weights();
	printf( "Type Y to continue: " );
	scanf( "%s", pattern[ 0 ] );

	for ( ; ; ) {
		printf( "\n\nWhich pattern (start line with ! to stop)? " );
		scanf( "%s", retr_pattern );
		if ( *retr_pattern == '!' )
			break;
		convert_pat( retr_pattern );
		printf( "[R]etrieve, test for [S]table, or [C]oose"
				  " firing sequence? " );
		scanf( "%s", resp );
		if ( *resp == 'R' ) {
			get_pattern( retr_pattern );
			printf( "Closest pattern = %s\n", retr_pattern );
		}
		else if ( *resp == 'S' )
			test_stable( retr_pattern );
		else
			choose_fire_seq( retr_pattern );
	}

	return EXIT_SUCCESS;
}

void store_pat( signed char pattern[][ MAXSIZE + 1], int no_pats )
{
	int i, j, p;

	for ( i = 0; i < no_nodes; i++ )
		for ( j = 0; j < no_nodes; j++ ) {
			weight[ i ][ j ] = 0;
			if ( i == j )
				continue;
			for ( p = 0; p < no_pats; p++ )
				weight[ i ][ j ] += pattern[ p ][ i ] * pattern[ p ][ j ];
		}
}

void convert_pat( signed char *p )
{
	int i;
	for ( i = 0; i < no_nodes; i++ )
		if ( p[ i ] == '+' )
			p[ i ] = 1;
		else
			p[ i ] = -1;
}

void get_pattern( signed char *retr_pattern )
{
	int i, j, firenode, sum;

	for ( i = 0; i < NUMB_FIRES; i++ ) {
		/* fire random node numbers firenode */
		firenode = rand() % no_nodes;
		sum = 0;
		for ( j = 0; j < no_nodes; j++ )
			sum += weight[ firenode ][ j ] * retr_pattern[ j ];
		retr_pattern[ firenode ] = sum >= 0 ? 1 : -1;
	}

	for ( i = 0; i < no_nodes; i++ )
		if ( retr_pattern[ i ] == 1 )
			retr_pattern[ i ] = '+';
		else
			retr_pattern[ i ] = '-';
}

void print_weights( void )
{
		int i, j;

		for ( i = 0; i < no_nodes; i++ ) {
			for ( j = 0; j < no_nodes; j++ )
				printf( "%3d", weight[ i ][ j ] );
			putchar( '\n' );
		}
}

void test_stable( signed char *retr_pattern )
{
	int i, j, sum;

	for ( i = 0; i < no_nodes; i++ ) {
		sum = 0;
		for ( j = 0; j < no_nodes; j++ )
			sum += weight[ i ][ j ] * retr_pattern[ j ];
		if ( sum >= 0 && retr_pattern[ i ] == -1
			  || sum < 0 && retr_pattern[ i ] == 1 ) {
			printf( "Unstable at index %d\n", i );
			return;
		}
	}
	printf( "Stable\n" );
}

void choose_fire_seq( signed char *retr_pattern )
{
	int i, j, firenode, sum;

	for ( ; ; ) {
		printf( "Fire which node (-1 to stop)? " );
		scanf( "%d", &firenode );
		if ( firenode == -1 )
			return;
		sum = 0;
		for ( j = 0; j < no_nodes; j++ )
			sum += weight[ firenode ][ j ] * retr_pattern[ j ];
		retr_pattern[ firenode ] = sum >= 0 ? 1 : -1;
		for ( i = 0; i < no_nodes; i++ )
			if ( retr_pattern[ i ] == 1 )
				putchar( '+' );
			else
				putchar( '-' );
		putchar( '\n' );
	}
}