#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define TRUE 1
#define FALSE 0
#define MAX_PIX_SIZE   50
#define MAX_MASK_SIZE  11

int edge_mask;  /* TRUE if current operator is an edge detector
						 and FALSE otherwise */

float  pix[ MAX_PIX_SIZE ][ MAX_PIX_SIZE ],
		 mask1[ MAX_MASK_SIZE ][ MAX_MASK_SIZE ],
		 mask2[ MAX_MASK_SIZE ][ MAX_MASK_SIZE ];

float laplace[][ MAX_MASK_SIZE ] = 
	{ { -1, -1, -1 },
	  { -1,  8, -1 },
	  { -1, -1, -1 } };
float binomial[][ MAX_MASK_SIZE ] = 
	{ { 0.0625, 0.125, 0.0625 },
	  { 0.125,  0.25,  0.125 },
	  { 0.0625, 0.125, 0.0625 } };
float prewitt1[][ MAX_MASK_SIZE ] = 
	{ { -1, 0, 1 },
	  { -1, 0, 1},
	  { -1, 0, 1 } };
float prewitt2[][ MAX_MASK_SIZE ] = 
	{ {  1,  1,  1 },
	  {  0,  0,  0 },
	  { -1, -1, -1 } };
float sobel1[][ MAX_MASK_SIZE ] = 
	{ { -1, 0, 1 },
	  { -2, 0, 2 },
	  { -1, 0, 1 } };
float sobel2[][ MAX_MASK_SIZE ] = 
	{ {  1,  2,  1 },
	  {  0,  0,  0 },
	  { -1, -2, -1 } };
float roberts1[][ MAX_MASK_SIZE ] = 
	{ { -1, 0 },
	  {  0, 1 } };
float roberts2[][ MAX_MASK_SIZE ] = 
	{ {  0, 1 },
	  { -1, 0 } };
float sharpen[][ MAX_MASK_SIZE ] = 
	{ { -1, -1, -1 },
	  { -1,  9, -1},
	  { -1, -1, -1 } };

int pix_size, mask_size;
 
char buff[ 80 ];

int menu( void );
void enter_pix( void );
void enter_mask( float [][ MAX_MASK_SIZE ] );
void print_pix( void );
void print_mask( float [][ MAX_MASK_SIZE ] );
void convolve( void );
void edge_convolve( void );
void get_mask( float [][ MAX_MASK_SIZE ], float [][ MAX_MASK_SIZE ], int );  

main()
{
	int resp;

	do {
		switch ( resp = menu() ) {
		case 1:
			enter_pix();
			break;
		case 2:
			edge_mask = FALSE;
			enter_mask( mask1 );
			break;
		case 3:
			edge_mask = TRUE;
			enter_mask( mask1 );
			enter_mask( mask2 );
			break;
		case 4:
			print_pix();
			break;
		case 5:
			print_mask( mask1 );
			if ( edge_mask )
				print_mask( mask2 );
			break;
		case 6:
			if ( edge_mask )
				edge_convolve();
			else
				convolve();
			break;
		case 7:
			edge_mask = FALSE;
			get_mask( mask1, laplace, 3 );
			printf( "\nLaplacian stored\n" );
			break;
		case 8:
			edge_mask = FALSE;
			get_mask( mask1, sharpen, 3 );
			printf( "\nSharpener stored\n" );
			break;
		case 9:
			edge_mask = FALSE;
			get_mask( mask1, binomial, 3 );
			printf( "\nBinomial smoother stored\n" );
			break;
		case 10:
			edge_mask = TRUE;
			get_mask( mask1, prewitt1, 3 );
			get_mask( mask2, prewitt2, 3 );
			printf( "\nPrewitt stored\n" );
			break;
		case 11:
			edge_mask = TRUE;
			get_mask( mask1, sobel1, 3 );
			get_mask( mask2, sobel2, 3 );
			printf( "\nSobel stored\n" );
			break;
		case 12:
			edge_mask = TRUE;
			get_mask( mask1, roberts1, 2 );
			get_mask( mask2, roberts2, 2 );
			printf( "\nRoberts stored\n" );
			break;
		case 13:
			break;
		default:
			printf( "\nIllegal response.\n" );
			break;
		}
	} while ( resp != 13 );
}

int menu( void )
{
	printf( "\n\t 1 -- Enter a picture." );
	printf( "\n\t 2 -- Enter a non-edge detector mask." );
	printf( "\n\t 3 -- Enter an edge detector mask." );
	printf( "\n\t 4 -- Print current picture." );
	printf( "\n\t 5 -- Print current mask(s)." );
	printf( "\n\t 6 -- Compute and display convolution." );
	printf( "\n\t 7 -- Get Laplacian mask." );
	printf( "\n\t 8 -- Get sharpener mask." );
	printf( "\n\t 9 -- Get binomial smoother mask." );
	printf( "\n\t10 -- Get Prewitt's edge detector masks." );
	printf( "\n\t11 -- Get Sobel's edge detector masks." );
	printf( "\n\t12 -- Get Robert's edge detector masks." );
	printf( "\n\t13 -- End program." );
	printf( "\n\n\tYour choice: " );
	gets( buff );
	return atoi( buff );
}

void enter_pix( void )
{
	int i, j;

	printf( "\nEnter number of rows (= number of columns): " );
	gets( buff );
	pix_size = atoi( buff );
	
	for ( i = 0; i < pix_size; i++ ) {
		printf("\nEnter row %d (RETURN after each item)\n", i + 1 );
		for ( j = 0; j < pix_size; j++ ) {
			printf( "\nNext item: " );
			gets( buff );
			pix[ i ][ j ] = atof( buff );
		}
	}
}

void enter_mask( float mask[][ MAX_MASK_SIZE ] )
{
	int i, j;

	printf( "\nEnter number of rows (= number of columns): " );
	gets( buff );
	mask_size = atoi( buff );
	
	for ( i = 0; i < mask_size; i++ ) {
		printf("\nEnter row %d (RETURN after each item)\n", i + 1 );
		for ( j = 0; j < mask_size; j++ ) {
			printf( "\nNext item: " );
			gets( buff );
			mask[ i ][ j ] = atof( buff );
		}
	}
}

void print_pix( void )
{
	int i, j;

	for ( i = 0; i < pix_size; i++ ) {
		putchar( '\n' );
		for ( j = 0; j < pix_size; j++ )
			printf( "%.2f ", pix[ i ][ j ] );
	}
	putchar( '\n' );
}

void print_mask( float mask[][ MAX_MASK_SIZE ] )
{
	int i, j;

	for ( i = 0; i < mask_size; i++ ) {
		putchar( '\n' );
		for ( j = 0; j < mask_size; j++ )
			printf( "%.2f ", mask[ i ][ j ] );
	}
	putchar( '\n' );
}

/* Convolver for single mask only. */
void convolve( void )
{
	int i, j, k, l;
	float val;

	for ( i = 0; i <= pix_size - mask_size; i++ ) {
		putchar( '\n' );
		for ( j = 0; j <= pix_size - mask_size; j++ ) {
			val = 0.0;
			for ( k = 0; k < mask_size; k++ )
				for ( l = 0; l < mask_size; l++ )
					val += pix[ i + k ][j + l ]
							 * mask1[ k ][ l ];
			printf( "%.2f ", val );
		}
	}
}

/* Convolver for edge detector. */
void edge_convolve( void )
{
	int i, j, k, l;
	float val1, val2;

	for ( i = 0; i <= pix_size - mask_size; i++ ) {
		putchar( '\n' );
		for ( j = 0; j <= pix_size - mask_size; j++ ) {
			val1 = val2 = 0.0;
			for ( k = 0; k < mask_size; k++ )
				for ( l = 0; l < mask_size; l++ ) {
					val1 += pix[ i + k ][j + l ]
							  * mask1[ k ][ l ];
					val2 += pix[ i + k ][j + l ]
							  * mask2[ k ][ l ];
				}
			printf( "%.2f ", fabs( val1 ) + fabs( val2 ) );
		}
	}
}

void get_mask( float mask[][ MAX_MASK_SIZE ],
					float mask_type[][ MAX_MASK_SIZE ],
					int m_size )
{
	int i, j;

	mask_size = m_size;

	for ( i = 0; i < mask_size; i++ )
		for ( j = 0; j < mask_size; j++ )
			mask[ i ][ j ] = mask_type[ i ][ j ];
}
